"""
Freedom client application

@copyright: 2011 Nir Soffer <nirsof@gmail.com>, Erez Waisbard <waisbard@gmail.com>
@license: GNU GPL, see COPYING for details.
"""

import sys
import getpass
import logging
import time
import optparse

from freedom import imap
from freedom import page
from freedom import xmpp
from freedom import config
from freedom import event

IMAP_HOST = 'imap.gmail.com'
XMPP_HOST = ('talk.google.com', 5222)

def main(args):
    parser = make_parser()
    options, args = parser.parse_args(args, config)
    configure_logger(options)
    if not args:
        parser.error('email-address required')
    user = args[0]
    password = getpass.getpass('Password: ')
    if options.event_log:
        if not (options.event_server and options.event_url):
            parser.error('--event_server and --event_url required when running in event logging mode');
        event.init(options.event_log)
        tester = xmpp.tester(XMPP_HOST, user, password, options.event_server, options.event_url, options.event_requests)
        tester.start()
    else:
        tester = None
    opener = page.opener(tester)
    opener.start()
    listener = imap.response_listener(IMAP_HOST, user, password, opener) 
    listener.start()
    try:
        while 1:
            time.sleep(60)
    except KeyboardInterrupt:
        pass
    if options.event_log:
        event.shutdown()
    logging.shutdown()

def make_parser():
    parser = optparse.OptionParser(usage='%prog [options] email-address')
    parser.add_option('-d', '--debug', dest='debug', action='store_true',
            help='show debug log message and keep temporary page directories (default %s).' % config.debug)
    parser.add_option('--event-log', dest='event_log',
            help='enable event logging mode, using given file. You must run the client and the server on the same machine and use the same log file in both.')
    parser.add_option('--event-server', dest='event_server',
            help='freedom server to connect to when running in event logging mode')
    parser.add_option('--event-url', dest='event_url',
            help='url to fetch when running in event logging mode')
    parser.add_option('--event-requests', dest='event_requests', type='int',
            help='number of request to send when running in event loging mode')
    return parser

def configure_logger(options):
    if options.debug:
        level = logging.DEBUG
    else:
        level = logging.INFO
    logging.basicConfig(level=level,
            format='[%(asctime)s] [%(levelname)s] %(message)s')

if __name__ ==  '__main__':
    args = sys.argv[1:]
    main(args)

