"""
Freedom document type detection 

See http://www.w3.org/QA/2002/04/valid-dtd-list

@copyright: 2011 Nir Soffer <nirsof@gmail.com>, Erez Waisbard <waisbard@gmail.com>
@license: GNU GPL, see COPYING for details.
"""

import sgmllib
import re
import logging

XHTML_PUBLIC_IDS = [
    '-//W3C//DTD XHTML 1.1//EN',
    '-//W3C//DTD XHTML 1.0 Strict//EN',
    '-//W3C//DTD XHTML 1.0 Transitional//EN',
    '-//W3C//DTD XHTML 1.0 Frameset//EN',
    '-//W3C//DTD XHTML 1.1 plus MathML 2.0 plus SVG 1.1//EN',
    '-//W3C//DTD XHTML Basic 1.1//EN',
    '-//W3C//DTD XHTML Basic 1.0//EN',
]

dtd_re = re.compile(r'''
DOCTYPE
\s+
(?P<doctype>\S+)
(?:
    \s+
    (?:
        (?:
            PUBLIC
            \s+
            (?P<public_id>".*?"|'.*?')
            (?:
                \s+
                (?P<public_uri>".*?"|'.*?')
            )?
        )
        |
        (?:
            SYSTEM
            \s+
            (?P<system_uri>".*?"|'.*?')
        )
    )
)?
''', re.VERBOSE | re.IGNORECASE)

def parse(data):
    dtd = parser()
    try:
        dtd.feed(data)
    except sgmllib.SGMLParseError, e:
        # Seems that SGMLParser has trouble parsing rest of the doctype, after
        # calling handle_decl.
        # XXX I leave this as wanring to get more info from real pages.
        logging.warning('error parsing doctype: %s', e)
    dtd.reset()
    return dtd

class parser(sgmllib.SGMLParser):

    def __init__(self):
        self.doctype = None
        self.public_id = None
        self.uri = None
        sgmllib.SGMLParser.__init__(self)

    def handle_decl(self, data):
        match = dtd_re.match(data)
        if match:
            d = match.groupdict()
            self.doctype = lower(d['doctype'])
            if d['public_id']:
                self.public_id = unquote(d['public_id'])
                self.uri = unquote(d['public_uri'])
            else:
                self.uri = unquote(d['system_uri'])

    def isxml(self):
        return self.public_id in XHTML_PUBLIC_IDS

def lower(s):
    if not s:
        return s
    return s.lower()

def unquote(s):
    if not s:
        return s
    if (s[0] == '"' and s[-1] == '"') or (s[0] == "'" and s[-1] == "'"):
        s = s[1:-1]
    return s

