import threading

from freedom import dispatch

class queue_delegate:
    def __init__(self):
        self.log = []
        self.started = threading.Event()
        self.handled = threading.Event()
        self.stopped = threading.Event()
    def queue_start(self):
        self.started.set()
    def queue_handle(self, msg):
        self.log.append(msg)
        self.handled.set()
    def queue_stop(self):
        self.stopped.set()

def test_queue_start():
    delegate = queue_delegate()
    queue = dispatch.queue(delegate)
    queue.start()
    assert delegate.started.wait(0.1)
     
def test_queue_handle():
    delegate = queue_delegate()
    queue = dispatch.queue(delegate)
    queue.start()
    delegate.started.wait(0.1)
    msg = object()
    queue.handle(msg)
    assert delegate.handled.wait(0.1)
    assert delegate.log == [msg]
    queue.stop()
    
def test_queue_stop():
    delegate = queue_delegate()
    queue = dispatch.queue(delegate)
    queue.start()
    delegate.started.wait(0.1)
    queue.stop()
    assert delegate.stopped.wait(0.1)
    
def test_queue_stop_clear_pending():
    delegate = queue_delegate()
    queue = dispatch.queue(delegate)
    queue.start()
    delegate.started.wait(0.1)
    queue.stop()
    msg = object()
    queue.handle(msg)
    delegate.stopped.wait(0.1)
    assert delegate.log == []
    assert not queue._queue
    
